/*
 * cvtbeacon.c -- (tries to) converts 23C3 Sputnik log file to text
 * (c) Peter Meerwald, pmeerw@pmeerw.net
 *
 * history:
 *   0.1 initial version
 *   0.2 cleanup, naming previously unknown fields
 *
 * This file is distributed under GPL license.
 */

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/param.h>
#include <string.h>

static char *progname = "cvtbeacon";
static char fname[MAXPATHLEN];

#define BEACON_FLAG_BUTTON 0x02

typedef struct {
    u_int32_t timestamp;
    u_int32_t sourceIP;
    u_int8_t unknown1, unknown2, unknown3, unknown4;
    u_int32_t sourceIP_reversed;	// wtf?
    u_int8_t unknown5;
    u_int8_t protocol;
    u_int8_t flags;
    u_int8_t signal_strength;
    u_int32_t seqno;
} __attribute__ ((packed)) beacon_log_t;

static int log_packet(beacon_log_t * bl, unsigned int packetno)
{
    struct in_addr sourceIP;

    if (!bl)
	return -1;

    if (bl->unknown1 != 0x02)
	return -2;

    if (bl->unknown2 != 0x00)
	return -3;

    sourceIP.s_addr = ntohl(bl->sourceIP);
    if (sourceIP.s_addr != bl->sourceIP_reversed)
	return -4;

    if (bl->unknown3 != 0x79)
	return -5;

    if (bl->unknown4 != 0x41)
	return -6;

    if (bl->unknown5 != 0x10)
	return -7;

    if (bl->protocol != 0x17)
	return -8;

    if (bl->flags != 0x00 && bl->flags != 0x02) {
	return -9;
    }

    if (bl->signal_strength != 0xff && bl->signal_strength != 0xaa
	&& bl->signal_strength != 0x55 && bl->signal_strength != 0x00) {
	return -10;
    }

    printf("%08x %s %08x %02x %02x\n",
	   ntohl(bl->timestamp),
	   inet_ntoa(sourceIP), ntohl(bl->seqno), bl->flags,
	   bl->signal_strength);

    return 1;
}

int main(int argc, char *argv[])
{
    FILE *f;
    unsigned char buf[sizeof(beacon_log_t)];
    size_t nread;
    unsigned int packetno;
    unsigned int rejected_packets;
    int reason;

    // sanity check
    assert(sizeof(beacon_log_t) == 0x18);

    if (argc == 1 || (argc == 2 && !strcmp(argv[1], "-"))) {
	f = stdin;
	strcpy(fname, "<STDIN>");
    } else {
	strcpy(fname, argv[1]);
	f = fopen(fname, "rb");
	if (!f) {
	    fprintf(stderr, "%s: failed to open '%s', exit.\n", progname,
		    fname);
	    exit(EXIT_FAILURE);
	}
    }

    rejected_packets = packetno = 0;
    while (!feof(f)) {

	nread = fread(buf, 1, sizeof(beacon_log_t), f);
	if (nread == 0) {
	    continue;
	} else if (nread != sizeof(beacon_log_t)) {
	    fprintf(stderr, "%s: failed to read data, stop.\n", progname);
	    break;
	}

	if ((reason = log_packet((beacon_log_t *) buf, packetno)) < 0) {
	    rejected_packets++;
	    fprintf(stderr,
		    "%s: rejected packet at offset %08lx (reason %d)...\n",
		    progname, ftell(f) - sizeof(beacon_log_t), reason);
	}

	packetno++;
    }

    fprintf(stderr, "%s: got %d packets (%d rejected)...\n", progname,
	    packetno, rejected_packets);

    fclose(f);

    return EXIT_SUCCESS;
}
